@@ -262,8 +262,9 @@ void parse_options_set_use_sat_parser(Parse_Options opts, bool dummy) {
 #ifdef USE_SAT_SOLVER
 	opts->use_sat_solver = dummy;
 #else
-	prt_error("Error: cannot enable the Boolean SAT parser; this "
-	          " library was built without SAT solver support.\n");
+	if (dummy)
+		prt_error("Error: cannot enable the Boolean SAT parser; this"
+					 " library was built without SAT solver support.");
 #endif
 }
 
