@@ -20,6 +20,7 @@
 #include "dict-common.h"
 #include "disjunct-utils.h"
 #include "externs.h"
+#include "string-set.h"
 #include "word-utils.h"
 #include "utilities.h" /* For Win32 compatibility features */
 
@@ -302,6 +303,7 @@ build_disjunct(Clause * cl, const char * string, double cost_cutoff)
 			ndis->string = string;
 			ndis->cost = cl->cost;
 			ndis->next = dis;
+			ndis->word = NULL;
 			dis = ndis;
 		}
 	}
@@ -322,7 +324,7 @@ Disjunct * build_disjuncts_for_exp(Exp* exp, const char *word, double cost_cutof
 }
 
 #if DEBUG
-/* There is a much better print_expression elsewhere 
+/* There is a much better print_expression elsewhere
  * This one is for low-level debug. */
 void prt_exp(Exp *e, int i)
 {
@@ -345,6 +347,41 @@ void prt_exp(Exp *e, int i)
 		printf("con=%s\n", e->u.string);
 	}
 }
+
+void prt_exp_mem(Exp *e, int i)
+{
+	const char *type;
+
+	if (e == NULL) return;
+
+	if (e->type > 0 && e->type < 4)
+	{
+		type = ((const char *[]) {"OR_type", "AND_type", "CONNECTOR_type"}) [e->type-1];
+	}
+	else
+	{
+		type = "unknown";
+	}
+
+	for(int j =0; j<i; j++) printf(" ");
+	printf ("e=%p: type=%d (%s) dir=%c multi=%d cost=%f\n", e, e->type, type, e->dir, e->multi, e->cost);
+	if (e->type != CONNECTOR_type)
+	{
+		E_list *l = e->u.l;
+		for(int j =0; j<i+2; j++) printf(" ");
+		printf("E_list=%p\n", l);
+		while(l)
+		{
+			prt_exp_mem(l->e, i+2);
+			l = l->next;
+		}
+	}
+	else
+	{
+		for(int j =0; j<i; j++) printf(" ");
+		printf("con=%s\n", e->u.string);
+	}
+}
 #endif
 
 /**
@@ -396,20 +433,25 @@ unsigned int count_disjunct_for_dict_node(Dict_node *dn)
 }
 
 /**
- * build_word_expressions() -- build list of expressions for a word
+ * build_word_expressions() -- build list of expressions for a word.
  *
- * Looks up the word s in the dictionary.  Returns NULL if it's not there.
- * If there, it builds the list of expressions for the word, and returns
- * a pointer to it.
+ * Looks up a word in the dictionary, fetching from it matching words and their
+ * expressions.  Returns NULL if it's not there.  If there, it builds the list
+ * of expressions for the word, and returns a pointer to it.
+ * The subword of Gword w is used for this lookup, unless the subword is
+ * explicitly given as parameter s. The subword of Gword w is always used as
+ * the base word for each expression, and its subscript is the one from the
+ * dictionary word of the expression.
  */
-X_node * build_word_expressions(Dictionary dict, const char * s)
+X_node * build_word_expressions(Sentence sent, const Gword *w, const char *s)
 {
 	Dict_node * dn, *dn_head;
 	X_node * x, * y;
 	Exp_list eli;
-	eli.exp_list = NULL;
+	const Dictionary dict = sent->dict;
 
-	dn_head = dictionary_lookup_list(dict, s);
+	eli.exp_list = NULL;
+	dn_head = dictionary_lookup_list(dict, NULL == s ? w->subword : s);
 	x = NULL;
 	dn = dn_head;
 	while (dn != NULL)
@@ -418,7 +460,21 @@ X_node * build_word_expressions(Dictionary dict, const char * s)
 		y->next = x;
 		x = y;
 		x->exp = copy_Exp(add_empty_word(dict, &eli, dn));
-		x->string = dn->string;
+		if (NULL == s)
+		{
+			x->string = dn->string;
+		}
+		else
+		{
+			dyn_str *xs = dyn_str_new();
+			const char *sm = strrchr(dn->string, SUBSCRIPT_MARK);
+
+			dyn_strcat(xs, w->subword);
+			if (NULL != sm) dyn_strcat(xs, sm);
+			x->string = string_set_add(xs->str, sent->string_set);
+			dyn_str_delete(xs);
+		}
+		x->word = w;
 		dn = dn->right;
 	}
 	free_lookup_list (dict, dn_head);
@@ -441,6 +497,7 @@ void build_sentence_disjuncts(Sentence sent, double cost_cutoff)
 		for (x = sent->word[w].x; x != NULL; x = x->next)
 		{
 			Disjunct *dx = build_disjuncts_for_exp(x->exp, x->string, cost_cutoff);
+			word_record_in_disjunct(x->word, dx);
 			d = catenate_disjuncts(dx, d);
 		}
 		sent->word[w].d = d;
