@@ -1,7 +1,7 @@
 /*************************************************************************/
 /* Copyright (c) 2004                                                    */
 /* Daniel Sleator, David Temperley, and John Lafferty                    */
-/* Copyright (c) 2013,2014 Linas Vepstas                                 */
+/* Copyright (c) 2013,2014,2015 Linas Vepstas                            */
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
@@ -29,10 +29,9 @@ struct Table_connector_s
 {
 	Table_connector  *next;
 	Connector        *le, *re;
-	s64              count;
+	Count_bin        count;
 	short            lw, rw;
-	unsigned short   cost; /* Cost, here and below, is actually the
-	                        * null-count being considered. */
+	unsigned short   null_count;
 };
 
 struct count_context_s
@@ -78,8 +77,8 @@ static void init_table(count_context_t *ctxt, size_t sent_len)
 
 	if (sent_len >= 10)
 	{
-		shift = 12 + (sent_len) / 6 ; 
-	} 
+		shift = 12 + (sent_len) / 4 ;
+	}
 	else
 	{
 		shift = 12;
@@ -89,7 +88,7 @@ static void init_table(count_context_t *ctxt, size_t sent_len)
 	if (24 < shift) shift = 24;
 	ctxt->table_size = (1U << shift);
 	ctxt->log2_table_size = shift;
-	ctxt->table = (Table_connector**) 
+	ctxt->table = (Table_connector**)
 		xalloc(ctxt->table_size * sizeof(Table_connector*));
 	memset(ctxt->table, 0, ctxt->table_size*sizeof(Table_connector*));
 }
@@ -106,41 +105,41 @@ bool do_match(Connector *a, Connector *b, int aw, int bw)
 	return easy_match(a->string, b->string);
 }
 
-/** 
+/**
  * Stores the value in the table.  Assumes it's not already there.
  */
 static Table_connector * table_store(count_context_t *ctxt,
                                      int lw, int rw,
                                      Connector *le, Connector *re,
-                                     unsigned int cost, s64 count)
+                                     unsigned int null_count)
 {
 	Table_connector *t, *n;
 	unsigned int h;
 
 	n = (Table_connector *) xalloc(sizeof(Table_connector));
-	n->count = count;
-	n->lw = lw; n->rw = rw; n->le = le; n->re = re; n->cost = cost;
-	h = pair_hash(ctxt->log2_table_size,lw, rw, le, re, cost);
+	n->lw = lw; n->rw = rw; n->le = le; n->re = re; n->null_count = null_count;
+	h = pair_hash(ctxt->table_size, lw, rw, le, re, null_count);
 	t = ctxt->table[h];
 	n->next = t;
 	ctxt->table[h] = n;
+
 	return n;
 }
 
 /** returns the pointer to this info, NULL if not there */
-static Table_connector * 
+static Table_connector *
 find_table_pointer(count_context_t *ctxt,
-                   int lw, int rw, 
+                   int lw, int rw,
                    Connector *le, Connector *re,
-                   unsigned int cost)
+                   unsigned int null_count)
 {
 	Table_connector *t;
-	unsigned int h = pair_hash(ctxt->log2_table_size,lw, rw, le, re, cost);
+	unsigned int h = pair_hash(ctxt->table_size,lw, rw, le, re, null_count);
 	t = ctxt->table[h];
 	for (; t != NULL; t = t->next) {
 		if ((t->lw == lw) && (t->rw == rw)
 		    && (t->le == le) && (t->re == re)
-		    && (t->cost == cost))  return t;
+		    && (t->null_count == null_count))  return t;
 	}
 
 	/* Create a new connector only if resources are exhausted.
@@ -151,54 +150,69 @@ find_table_pointer(count_context_t *ctxt,
 	 */
 	ctxt->checktimer ++;
 	if (ctxt->exhausted || ((0 == ctxt->checktimer%450100) &&
-	                       (ctxt->current_resources != NULL) && 
+	                       (ctxt->current_resources != NULL) &&
 	                       resources_exhausted(ctxt->current_resources)))
 	{
 		ctxt->exhausted = true;
-		return table_store(ctxt, lw, rw, le, re, cost, 0);
+		t = table_store(ctxt, lw, rw, le, re, null_count);
+		t->count = hist_zero();
+		return t;
 	}
 	else return NULL;
 }
 
 /** returns the count for this quintuple if there, -1 otherwise */
-s64 table_lookup(count_context_t * ctxt, 
-                 int lw, int rw, Connector *le, Connector *re, unsigned int cost)
+Count_bin* table_lookup(count_context_t * ctxt,
+                       int lw, int rw, Connector *le, Connector *re,
+                       unsigned int null_count)
 {
-	Table_connector *t = find_table_pointer(ctxt, lw, rw, le, re, cost);
+	Table_connector *t = find_table_pointer(ctxt, lw, rw, le, re, null_count);
 
-	if (t == NULL) return -1; else return t->count;
+	if (t == NULL) return NULL; else return &t->count;
 }
 
 /**
- * Returns 0 if and only if this entry is in the hash table 
- * with a count value of 0.
+ * psuedocount is used to check to see if a parse is even possible,
+ * so that we don't waste cpu time performing an actual count, only
+ * to discover that it is zero.
+ *
+ * Returns false if and only if this entry is in the hash table
+ * with a count value of 0. If an entry is not in the hash table,
+ * we have to assume the worst case: that the count might be non-zero,
+ * and since we don't know, we return true.  However, if the entry is
+ * in the hash table, and its zero, then we know, for sure, that the
+ * count is zero.
  */
-static s64 pseudocount(count_context_t * ctxt,
+static bool pseudocount(count_context_t * ctxt,
                        int lw, int rw, Connector *le, Connector *re,
-                       unsigned int cost)
+                       unsigned int null_count)
 {
-	s64 count = table_lookup(ctxt, lw, rw, le, re, cost);
-	if (count == 0) return 0; else return 1;
+	Count_bin * count = table_lookup(ctxt, lw, rw, le, re, null_count);
+	if (NULL == count) return true;
+	if (hist_total(count) == 0) return false;
+	return true;
 }
 
-static s64 do_count(fast_matcher_t *mchxt, 
-                    count_context_t *ctxt,
-                    int lw, int rw,
-                    Connector *le, Connector *re, int null_count)
+static Count_bin do_count(fast_matcher_t *mchxt,
+                          count_context_t *ctxt,
+                          int lw, int rw,
+                          Connector *le, Connector *re,
+                          int null_count)
 {
-	s64 total;
+	Count_bin zero = hist_zero();
+	Count_bin total;
 	int start_word, end_word, w;
 	Table_connector *t;
 
-	if (null_count < 0) return 0;  /* can this ever happen?? */
+	assert (0 <= null_count, "Bad null count");
 
 	t = find_table_pointer(ctxt, lw, rw, le, re, null_count);
 
 	if (t) return t->count;
 
-	/* Create the table entry with a tentative null count of 0. 
+	/* Create the table entry with a tentative null count of 0.
 	 * This count must be updated before we return. */
-	t = table_store(ctxt, lw, rw, le, re, null_count, 0);
+	t = table_store(ctxt, lw, rw, le, re, null_count);
 
 	if (rw == 1+lw)
 	{
@@ -206,15 +220,17 @@ static s64 do_count(fast_matcher_t *mchxt,
 		/* You can't have a linkage here with null_count > 0 */
 		if ((le == NULL) && (re == NULL) && (null_count == 0))
 		{
-			t->count = 1;
+			t->count = hist_one();
 		}
 		else
 		{
-			t->count = 0;
+			t->count = zero;
 		}
 		return t->count;
 	}
 
+	/* The left and right connectors are null, but the two words are
+	 * NOT next to each-other. */
 	if ((le == NULL) && (re == NULL))
 	{
 		if (!ctxt->islands_ok && (lw != -1))
@@ -222,17 +238,13 @@ static s64 do_count(fast_matcher_t *mchxt,
 			/* If we don't allow islands (a set of words linked together
 			 * but separate from the rest of the sentence) then the
 			 * null_count of skipping n words is just n. */
-			/* null_block is always 1 these days. */
-			/* if (null_count == ((rw-lw-1) + ctxt->null_block-1) / ctxt->null_block) */
 			if (null_count == (rw-lw-1))
 			{
-				/* If null_block=4 then the null_count of
-				   1,2,3,4 nulls is 1; and 5,6,7,8 is 2 etc. */
-				t->count = 1;
+				t->count = hist_one();
 			}
 			else
 			{
-				t->count = 0;
+				t->count = zero;
 			}
 			return t->count;
 		}
@@ -241,22 +253,23 @@ static s64 do_count(fast_matcher_t *mchxt,
 			/* There is no solution without nulls in this case. There is
 			 * a slight efficiency hack to separate this null_count==0
 			 * case out, but not necessary for correctness */
-			t->count = 0;
+			t->count = zero;
 		}
 		else
 		{
+			t->count = zero;
 			Disjunct * d;
-			s64 total = 0;
 			int w = lw + 1;
 			for (d = ctxt->local_sent[w].d; d != NULL; d = d->next)
 			{
 				if (d->left == NULL)
 				{
-					total += do_count(mchxt, ctxt, w, rw, d->right, NULL, null_count-1);
+					hist_accumv(&t->count, d->cost,
+						do_count(mchxt, ctxt, w, rw, d->right, NULL, null_count-1));
 				}
 			}
-			total += do_count(mchxt, ctxt, w, rw, NULL, NULL, null_count-1);
-			t->count = total;
+			hist_accumv(&t->count, 0.0,
+				do_count(mchxt, ctxt, w, rw, NULL, NULL, null_count-1));
 		}
 		return t->count;
 	}
@@ -279,7 +292,7 @@ static s64 do_count(fast_matcher_t *mchxt,
 		end_word = re->word +1;
 	}
 
-	total = 0;
+	total = zero;
 
 	for (w = start_word; w < end_word; w++)
 	{
@@ -287,89 +300,133 @@ static s64 do_count(fast_matcher_t *mchxt,
 		m1 = m = form_match_list(mchxt, w, le, lw, re, rw);
 		for (; m != NULL; m = m->next)
 		{
-			unsigned int lcost, rcost;
-			unsigned int null_count_p1;
-			Disjunct * d;
-			d = m->d;
-			null_count_p1 = null_count + 1; /* avoid gcc warning: unsafe loop opt */
-			for (lcost = 0; lcost < null_count_p1; lcost++)
+			unsigned int lnull_cnt, rnull_cnt;
+			Disjunct * d = m->d;
+			/* _p1 avoids a gcc warning about unsafe loop opt */
+			unsigned int null_count_p1 = null_count + 1;
+
+			for (lnull_cnt = 0; lnull_cnt < null_count_p1; lnull_cnt++)
 			{
 				bool Lmatch, Rmatch;
-				s64 leftcount = 0, rightcount = 0;
-				s64 pseudototal;
+				bool leftpcount = false;
+				bool rightpcount = false;
+				bool pseudototal = false;
 
-				rcost = null_count - lcost;
-				/* Now lcost and rcost are the costs we're assigning
+				rnull_cnt = null_count - lnull_cnt;
+				/* Now lnull_cnt and rnull_cnt are the costs we're assigning
 				 * to those parts respectively */
 
 				/* Now, we determine if (based on table only) we can see that
 				   the current range is not parsable. */
-				Lmatch = (le != NULL) && (d->left != NULL) && 
+				Lmatch = (le != NULL) && (d->left != NULL) &&
 				         do_match(le, d->left, lw, w);
-				Rmatch = (d->right != NULL) && (re != NULL) && 
+				Rmatch = (d->right != NULL) && (re != NULL) &&
 				         do_match(d->right, re, w, rw);
 
+				/* First, perform pseudocounting as an optimization. If
+				 * the pseudocount is zero, then we know that the true
+				 * count will be zero, and so skip counting entirely,
+				 * in that case.
+				 */
 				if (Lmatch)
 				{
-					leftcount = pseudocount(ctxt, lw, w, le->next, d->left->next, lcost);
-					if (le->multi) leftcount += pseudocount(ctxt, lw, w, le, d->left->next, lcost);
-					if (d->left->multi) leftcount += pseudocount(ctxt, lw, w, le->next, d->left, lcost);
-					if (le->multi && d->left->multi) leftcount += pseudocount(ctxt, lw, w, le, d->left, lcost);
+					leftpcount = pseudocount(ctxt, lw, w, le->next, d->left->next, lnull_cnt);
+					if (!leftpcount && le->multi)
+						leftpcount =
+							pseudocount(ctxt, lw, w, le, d->left->next, lnull_cnt);
+					if (!leftpcount && d->left->multi)
+						leftpcount =
+							pseudocount(ctxt, lw, w, le->next, d->left, lnull_cnt);
+					if (!leftpcount && le->multi && d->left->multi)
+						leftpcount =
+							pseudocount(ctxt, lw, w, le, d->left, lnull_cnt);
 				}
 
 				if (Rmatch)
 				{
-					rightcount = pseudocount(ctxt, w, rw, d->right->next, re->next, rcost);
-					if (d->right->multi) rightcount += pseudocount(ctxt, w,rw,d->right,re->next, rcost);
-					if (re->multi) rightcount += pseudocount(ctxt, w, rw, d->right->next, re, rcost);
-					if (d->right->multi && re->multi) rightcount += pseudocount(ctxt, w, rw, d->right, re, rcost);
+					rightpcount = pseudocount(ctxt, w, rw, d->right->next, re->next, rnull_cnt);
+					if (!rightpcount && d->right->multi)
+						rightpcount =
+							pseudocount(ctxt, w,rw, d->right, re->next, rnull_cnt);
+					if (!rightpcount && re->multi)
+						rightpcount =
+							pseudocount(ctxt, w, rw, d->right->next, re, rnull_cnt);
+					if (!rightpcount && d->right->multi && re->multi)
+						rightpcount =
+							pseudocount(ctxt, w, rw, d->right, re, rnull_cnt);
 				}
 
-				/* total number where links are used on both sides */
-				pseudototal = leftcount*rightcount;
+				/* Total number where links are used on both sides */
+				pseudototal = leftpcount && rightpcount;
 
-				if (leftcount > 0) {
-					/* evaluate using the left match, but not the right */
-					pseudototal += leftcount * pseudocount(ctxt, w, rw, d->right, re, rcost);
+				if (!pseudototal && leftpcount) {
+					/* Evaluate using the left match, but not the right. */
+					pseudototal =
+						pseudocount(ctxt, w, rw, d->right, re, rnull_cnt);
 				}
-				if ((le == NULL) && (rightcount > 0)) {
-					/* evaluate using the right match, but not the left */
-					pseudototal += rightcount * pseudocount(ctxt, lw, w, le, d->left, lcost);
+				if (!pseudototal && (le == NULL) && rightpcount) {
+					/* Evaluate using the right match, but not the left. */
+					pseudototal =
+						pseudocount(ctxt, lw, w, le, d->left, lnull_cnt);
 				}
 
-				/* now pseudototal is 0 implies that we know that the true total is 0 */
-				if (pseudototal != 0) {
-					rightcount = leftcount = 0;
+				/* If pseudototal is zero (false), that implies that
+				 * we know that the true total is zero. So we don't
+				 * bother counting at all, in that case. */
+				if (pseudototal)
+				{
+					Count_bin leftcount = zero;
+					Count_bin rightcount = zero;
 					if (Lmatch) {
-						leftcount = do_count(mchxt, ctxt, lw, w, le->next, d->left->next, lcost);
-						if (le->multi) leftcount += do_count(mchxt, ctxt, lw, w, le, d->left->next, lcost);
-						if (d->left->multi) leftcount += do_count(mchxt, ctxt, lw, w, le->next, d->left, lcost);
-						if (le->multi && d->left->multi) leftcount += do_count(mchxt, ctxt, lw, w, le, d->left, lcost);
+						leftcount = do_count(mchxt, ctxt, lw, w, le->next, d->left->next, lnull_cnt);
+						if (le->multi)
+							hist_accumv(&leftcount, d->cost,
+								do_count(mchxt, ctxt, lw, w, le, d->left->next, lnull_cnt));
+						if (d->left->multi)
+							hist_accumv(&leftcount, d->cost,
+								 do_count(mchxt, ctxt, lw, w, le->next, d->left, lnull_cnt));
+						if (le->multi && d->left->multi)
+							hist_accumv(&leftcount, d->cost,
+								do_count(mchxt, ctxt, lw, w, le, d->left, lnull_cnt));
 					}
 
 					if (Rmatch) {
-						rightcount = do_count(mchxt, ctxt, w, rw, d->right->next, re->next, rcost);
-						if (d->right->multi) rightcount += do_count(mchxt, ctxt, w, rw, d->right,re->next, rcost);
-						if (re->multi) rightcount += do_count(mchxt, ctxt, w, rw, d->right->next, re, rcost);
-						if (d->right->multi && re->multi) rightcount += do_count(mchxt, ctxt, w, rw, d->right, re, rcost);
+						rightcount = do_count(mchxt, ctxt, w, rw, d->right->next, re->next, rnull_cnt);
+						if (d->right->multi)
+							hist_accumv(&rightcount, d->cost,
+								do_count(mchxt, ctxt, w, rw, d->right,re->next, rnull_cnt));
+						if (re->multi)
+							hist_accumv(&rightcount, d->cost,
+								do_count(mchxt, ctxt, w, rw, d->right->next, re, rnull_cnt));
+						if (d->right->multi && re->multi)
+							hist_accumv(&rightcount, d->cost,
+								do_count(mchxt, ctxt, w, rw, d->right, re, rnull_cnt));
 					}
 
 					/* Total number where links are used on both sides */
-					total += leftcount*rightcount;
+					hist_muladd(&total, &leftcount, 0.0, &rightcount);
 
-					if (leftcount > 0) {
+					if (0 < hist_total(&leftcount))
+					{
 						/* Evaluate using the left match, but not the right */
-						total += leftcount * do_count(mchxt, ctxt, w, rw, d->right, re, rcost);
+						hist_muladdv(&total, &leftcount, d->cost,
+							do_count(mchxt, ctxt, w, rw, d->right, re, rnull_cnt));
 					}
-					if ((le == NULL) && (rightcount > 0)) {
+					if ((le == NULL) && (0 < hist_total(&rightcount)))
+					{
 						/* Evaluate using the right match, but not the left */
-						total += rightcount * do_count(mchxt, ctxt, lw, w, le, d->left, lcost);
+						hist_muladdv(&total, &rightcount, d->cost,
+							do_count(mchxt, ctxt, lw, w, le, d->left, lnull_cnt));
 					}
 
 					/* Sigh. Overflows can and do occur, esp for the ANY language. */
-					if (INT_MAX < total)
+					if (INT_MAX < hist_total(&total))
 					{
+#ifdef PERFORM_COUNT_HISTOGRAMMING
+						total.total = INT_MAX;
+#else
 						total = INT_MAX;
+#endif /* PERFORM_COUNT_HISTOGRAMMING */
 						t->count = total;
 						put_match_list(mchxt, m1);
 						return total;
@@ -384,10 +441,10 @@ static s64 do_count(fast_matcher_t *mchxt,
 }
 
 
-/** 
+/**
  * Returns the number of ways the sentence can be parsed with the
  * specified null count. Assumes that the fast-matcher and the count
- * context have already been initialized, and are freed later. The
+ * context have already been initialized, and will be freed later. The
  * "null_count" argument is the number of words that are allowed to
  * have no links to them.
  *
@@ -406,13 +463,19 @@ static s64 do_count(fast_matcher_t *mchxt,
  *
  * The count returned here is meant to be completely accurate; it is
  * not an approximation!
+ *
+ * Currently, the code has been designed to maintain a histogram of
+ * the cost of each of the parses. The number and width of the bins
+ * is adjustable in histogram.c. At this time, the histogram is not
+ * used anywhere, and a 3-5% speedup is available if it is avoided.
+ * We plan to use this historgram, later ....
  */
-s64 do_parse(Sentence sent,
-             fast_matcher_t *mchxt,
-             count_context_t *ctxt,
-             int null_count, Parse_Options opts)
+Count_bin do_parse(Sentence sent,
+                   fast_matcher_t *mchxt,
+                   count_context_t *ctxt,
+                   int null_count, Parse_Options opts)
 {
-	s64 total;
+	Count_bin hist;
 
 	ctxt->current_resources = opts->resources;
 	ctxt->exhausted = false;
@@ -424,12 +487,12 @@ s64 do_parse(Sentence sent,
 	/* ctxt->null_block = 1; */
 	ctxt->islands_ok = opts->islands_ok;
 
-	total = do_count(mchxt, ctxt, -1, sent->length, NULL, NULL, null_count+1);
+	hist = do_count(mchxt, ctxt, -1, sent->length, NULL, NULL, null_count+1);
 
 	ctxt->local_sent = NULL;
 	ctxt->current_resources = NULL;
 	ctxt->checktimer = 0;
-	return total;
+	return hist;
 }
 
 void delete_unmarked_disjuncts(Sentence sent)
