@@ -11,9 +11,7 @@
 /*                                                                       */
 /*************************************************************************/
 
-#ifdef USE_ANYSPLIT
 #include "anysplit.h"
-#endif
 #include "api-structures.h"
 #include "dict-api.h"
 #include "dict-common.h"
@@ -58,7 +56,7 @@ static inline char * deinflect(const char * str)
  * has a pointer to an array of strings which are the punctuation/affix
  * names. */
 
-const char *afdict_classname[] = { AFDICT_CLASSNAMES };
+const char * afdict_classname[] = { AFDICT_CLASSNAMES };
 #define NUMELEMS(a) (sizeof(a) / sizeof(a[0]))
 
 /**
@@ -102,7 +100,7 @@ static void affix_list_add(Dictionary afdict, Afdict_class * ac,
 
 static void load_affix(Dictionary afdict, Dict_node *dn, int l)
 {
-	Dict_node * dnx = NULL;	
+	Dict_node * dnx = NULL;
 	for (; NULL != dn; dn = dnx)
 	{
 		char *string;
@@ -155,7 +153,7 @@ static int revcmplen(const void *a, const void *b)
 /**
  * Traverse the main dict in dictionary order, and extract all the suffixes
  * and prefixes - every time we see a new suffix/prefix (the previous one is
- * remembered by w_last), we save it in the the corresponding affix-class list.
+ * remembered by w_last), we save it in the corresponding affix-class list.
  * The saved affixes don't include the infix mark.
  *
  * The empty word is not an affix so it is ignored.
@@ -312,11 +310,14 @@ static bool afdict_init(Dictionary dict)
 	}
 	/* XXX For now there is a possibility to use predefined SUF and PRE lists.
 	 * So if SUF or PRE are defined, don't extract any of them from the dict. */
-	if (1 == ac->length && (0 == AFCLASS(afdict, AFDICT_PRE)->length) &&
-			                 (0 == AFCLASS(afdict, AFDICT_SUF)->length))
+	if (1 == ac->length)
 	{
-		char last_entry[MAX_WORD+1] = "";
-		get_dict_affixes(dict, dict->root, ac->string[0][0], last_entry);
+		if ((0 == AFCLASS(afdict, AFDICT_PRE)->length) &&
+		    (0 == AFCLASS(afdict, AFDICT_SUF)->length))
+		{
+			char last_entry[MAX_WORD+1] = "";
+			get_dict_affixes(dict, dict->root, ac->string[0][0], last_entry);
+		}
 	}
 	else
 	{
@@ -348,13 +349,23 @@ static bool afdict_init(Dictionary dict)
 	{
 		int rc;
 
-		Regex_node * sm_re = (Regex_node *) malloc(sizeof(Regex_node));
+		Regex_node *sm_re = malloc(sizeof(*sm_re));
 		dyn_str *rebuf = dyn_str_new();
 
-		/* The regex is converted to: ^((original-regex)b)+$ */
+		/* The regex used to be converted to: ^((original-regex)b)+$
+		 * In the initial wordgraph version word boundaries are not supported,
+		 * so instead it is converted to: ^(original-regex)+$ */
+#ifdef WORD_BOUNDARIES
 		dyn_strcat(rebuf, "^((");
+#else
+		dyn_strcat(rebuf, "^(");
+#endif
 		dyn_strcat(rebuf, ac->string[0]);
+#ifdef WORD_BOUNDARIES
 		dyn_strcat(rebuf, ")b)+$");
+#else
+		dyn_strcat(rebuf, ")+$");
+#endif
 		sm_re->pattern = strdup(rebuf->str);
 		dyn_str_delete(rebuf);
 
@@ -362,6 +373,7 @@ static bool afdict_init(Dictionary dict)
 		sm_re->name = strdup(afdict_classname[AFDICT_SANEMORPHISM]);
 		sm_re->re = NULL;
 		sm_re->next = NULL;
+		sm_re->neg = false;
 		rc = compile_regexs(afdict->regex_root, afdict);
 		if (rc) {
 			prt_error("Error: afdict_init: Failed to compile "
@@ -399,9 +411,7 @@ static bool afdict_init(Dictionary dict)
 	if (! afdict_to_wide(afdict, AFDICT_QUOTES)) return false;
 	if (! afdict_to_wide(afdict, AFDICT_BULLETS)) return false;
 
-#ifdef USE_ANYSPLIT
 	if (! anysplit_init(afdict)) return false;
-#endif
 
 	return true;
 }
@@ -442,24 +452,6 @@ dictionary_six_str(const char * lang,
 	dict = (Dictionary) xalloc(sizeof(struct Dictionary_s));
 	memset(dict, 0, sizeof(struct Dictionary_s));
 
-	dict->num_entries = 0;
-	dict->is_special = false;
-	dict->already_got_it = '\0';
-	dict->line_number = 0;
-	dict->root = NULL;
-	dict->regex_root = NULL;
-	dict->word_file_header = NULL;
-	dict->exp_list.exp_list = NULL;
-	dict->affix_table = NULL;
-	dict->recursive_error = false;
-	dict->version = NULL;
-#ifdef HAVE_SQLITE
-	dict->db_handle = NULL;
-#endif
-#ifdef USE_ANYSPLIT
-	dict->anysplit = NULL;
-#endif
-
 	/* Language and file-name stuff */
 	dict->string_set = string_set_create();
 	t = strrchr (lang, '/');
@@ -479,6 +471,13 @@ dictionary_six_str(const char * lang,
 	{
 		/* To disable spell-checking, just set the checker to NULL */
 		dict->spell_checker = spellcheck_create(dict->lang);
+#if defined HAVE_HUNSPELL || defined HAVE_ASPELL
+		/* TODO:
+		 * 1. Set the spell option to 0, to signify no spell checking is done.
+		 * 2. On verbosity >= 1, add a detailed message on the reason. */
+		if (NULL == dict->spell_checker)
+			prt_error("Info: Spell checker disabled.");
+#endif
 		dict->insert_entry = insert_list;
 
 		dict->lookup_list = lookup_list;
@@ -557,11 +556,9 @@ dictionary_six_str(const char * lang,
 	dict->use_unknown_word = true;
 
 	dict_node = dictionary_lookup_list(dict, UNLIMITED_CONNECTORS_WORD);
-	if (dict_node != NULL) {
+	if (dict_node != NULL)
 		dict->unlimited_connector_set = connector_set_create(dict_node->exp);
-	} else {
-		dict->unlimited_connector_set = NULL;
-	}
+
 	free_lookup(dict_node);
 
 	return dict;
