@@ -15,12 +15,12 @@
 
 #include "link-includes.h"
 
-typedef struct 
+typedef struct
 {
 	Sentence sent;
 } err_ctxt;
 
-typedef enum 
+typedef enum
 {
 	Fatal = 1,
 	Error,
@@ -30,7 +30,7 @@ typedef enum
 } severity;
 
 void err_msg(err_ctxt *, severity, const char *fmt, ...) GNUC_PRINTF(3,4);
-bool feature_enabled(const char *, const char *);
+const char *feature_enabled(const char *, const char *);
 
 #ifdef _WIN32
 # define __func__ __FUNCTION__
@@ -45,15 +45,15 @@ bool feature_enabled(const char *, const char *);
  */
 #define lgdebug(level, ...) \
 (((verbosity >= (level)) && \
-	 (('\0' == debug[0]) || feature_enabled(debug, __func__))) \
-	 ? ((STRINGIFY(level)[0] == '+' ? printf("%s: ", __func__) : (void)0), \
-	 printf(__VA_ARGS__)) : (void)0)
+	(('\0' == debug[0]) || feature_enabled(debug, __func__))) \
+	? ((STRINGIFY(level)[0] == '+' ? (void)printf("%s: ", __func__) : (void)0), \
+	(void)printf(__VA_ARGS__)) : (void)0)
 
 /**
  * Return TRUE if the given feature (a string) is set in the !test variable
  * (a comma-separated feature list).
  */
 #define test_enabled(feature) \
-	(('\0' != test) && feature_enabled(test, feature))
+	(('\0' != test) ? feature_enabled(test, feature) : NULL)
 
 #endif
