@@ -57,7 +57,6 @@ static int right_disjunct_list_length(const Disjunct * d)
 struct fast_matcher_s
 {
 	size_t size;
-	unsigned int match_cost;     /* used for nothing but debugging ... */
 	unsigned int *l_table_size;  /* the sizes of the hash tables */
 	unsigned int *r_table_size;
 
@@ -120,7 +119,6 @@ void free_fast_matcher(fast_matcher_t *mchxt)
 	size_t w;
 	unsigned int i;
 
-	if (verbosity > 1) printf("%d Match cost\n", mchxt->match_cost);
 	for (w = 0; w < mchxt->size; w++)
 	{
 		for (i = 0; i < mchxt->l_table_size[w]; i++)
@@ -242,7 +240,6 @@ fast_matcher_t* alloc_fast_matcher(const Sentence sent)
 	ctxt->l_table = xalloc(2 * sent->length * sizeof(Match_node **));
 	ctxt->r_table = ctxt->l_table + sent->length;
 	memset(ctxt->l_table, 0, 2 * sent->length * sizeof(Match_node **));
-	ctxt->match_cost = 0;
 	ctxt->mn_free_list = NULL;
 
 	for (w=0; w<sent->length; w++)
@@ -285,18 +282,14 @@ fast_matcher_t* alloc_fast_matcher(const Sentence sent)
  * and rc came respectively.
  *
  * The list is returned in a linked list of Match_nodes.
- * The list contains no duplicates.  A quadratic algorithm is used to
- * eliminate duplicates.  In practice the match_cost is less than the
- * parse_cost (and the loop is tiny), so there's no reason to bother
- * to fix this.  The number of times through the loop is counted with
- * 'match_cost', if verbosity>1, then it this will be prnted at the end.
+ * The list contains no duplicates.
  */
 Match_node *
 form_match_list(fast_matcher_t *ctxt, int w,
                 Connector *lc, int lw,
                 Connector *rc, int rw)
 {
-	Match_node *ml, *mr, *mx, *my, *mz, *front, *free_later;
+	Match_node *ml, *mr, *mx, *my, *mr_end, *front;
 
 	if (lc != NULL) {
 		ml = ctxt->l_table[w][connector_hash(lc) & (ctxt->l_table_size[w]-1)];
@@ -309,56 +302,38 @@ form_match_list(fast_matcher_t *ctxt, int w,
 		mr = NULL;
 	}
 
-	front = NULL;
-	for (mx = ml; mx != NULL; mx = mx->next)
-	{
-		if (mx->d->left->word < lw) break;
-		my = get_match_node(ctxt);
-		my->d = mx->d;
-		my->next = front;
-		front = my;
-	}
-	ml = front;   /* ml is now the list of things that could match the left */
-
-	front = NULL;
+	/* In order to eliminate duplicates from the lists, we mark the disjuncts. */
 	for (mx = mr; mx != NULL; mx = mx->next)
 	{
 		if (mx->d->right->word > rw) break;
+		mx->d->marked = true;
+	}
+	mr_end = mx;
+
+	front = NULL;
+	/* Construct the list of things that could match the left. */
+	for (mx = ml; mx != NULL; mx = mx->next)
+	{
+		if (mx->d->left->word < lw) break;
+		mx->d->marked = false;
 		my = get_match_node(ctxt);
 		my->d = mx->d;
 		my->next = front;
 		front = my;
 	}
-	mr = front;   /* mr is now the list of things that could match the right */
 
-	/* Now we want to eliminate duplicates from the lists */
-	free_later = NULL;
-	front = NULL;
-	for (mx = mr; mx != NULL; mx = mz)
+	/* Append the list of things that could match the right. */
+	for (mx = mr; mx != mr_end; mx = mx->next)
 	{
-		/* see if mx in first list, put it in if its not */
-		mz = mx->next;
-		ctxt->match_cost++;
-		for (my=ml; my!=NULL; my=my->next) {
-			ctxt->match_cost++;
-			if (mx->d == my->d) break;
-		}
-		if (my != NULL) { /* mx was in the l list */
-			mx->next = free_later;
-			free_later = mx;
-		} else {  /* it was not there */
-			mx->next = front;
-			front = mx;
+		if (mx->d->marked)
+		{
+			/* mx is not in the l list. */
+			my = get_match_node(ctxt);
+			my->d = mx->d;
+			my->next = front;
+			front = my;
 		}
 	}
-	mr = front;  /* mr is now the abbreviated right list */
-	put_match_list(ctxt, free_later);
 
-	/* now catenate the two lists */
-	if (mr == NULL) return ml;
-	if (ml == NULL) return mr;
-	for (mx = mr; mx->next != NULL; mx = mx->next)
-	  ;
-	mx->next = ml;
-	return mr;
+	return front;
 }
