@@ -9,12 +9,15 @@
 /* forms, with or without modification, subject to certain conditions.   */
 /*                                                                       */
 /*************************************************************************/
+
+#include "api-types.h"
+
 bool separate_sentence(Sentence, Parse_Options);
-void build_sentence_expressions(Sentence, Parse_Options);
 bool sentence_in_dictionary(Sentence);
-#ifdef USE_ANYSPLIT
-void add_alternative(Sentence sent,
+bool flatten_wordgraph(Sentence, Parse_Options);
+
+Gword *issue_word_alternative(Sentence sent, Gword *unsplit_word,
+                     const char *label,
                      int prefnum, const char **prefix,
                      int stemnum, const char **stem,
                      int suffnum, const char **suffix);
-#endif
