@@ -5,7 +5,7 @@
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
-/* license set forth in the LICENSE file included with this software.    */ 
+/* license set forth in the LICENSE file included with this software.    */
 /* This license allows free redistribution and use in source and binary  */
 /* forms, with or without modification, subject to certain conditions.   */
 /*                                                                       */
@@ -27,13 +27,13 @@
 #include <string.h>
 
 #ifndef __CYGWIN__
-/* I was told that cygwin does not have these files. */ 
+/* I was told that cygwin does not have these files. */
 #include <wchar.h>
 #include <wctype.h>
 #endif
 
 #if defined(__CYGWIN__) && defined(__MINGW32__)
-/* Some users have CygWin and MinGW installed! 
+/* Some users have CygWin and MinGW installed!
  * In this case, use the MinGW versions of UTF-8 support. */
 #include <wchar.h>
 #include <wctype.h>
@@ -74,9 +74,20 @@ void *alloca (size_t);
 #define snprintf _snprintf
 #define vsnprintf _vsnprintf
 #define strcasecmp _stricmp
+#ifndef strdup
 #define strdup _strdup
+#endif
 #define strncasecmp(a,b,s) strnicmp((a),(b),(s))
 
+/* Microsoft does not support %zu -- it wants %Iu */
+int printf_compat(const char *, ...);
+int fprintf_compat(FILE *, const char *, ...);
+int sprintf_compat(char *, const char *, ...);
+
+#define fprintf fprintf_compat
+#define printf printf_compat
+#define sprintf sprintf_compat
+
 /* MS Visual C does not support some C99 standard floating-point functions */
 #define fmaxf(a,b) ((a) > (b) ? (a) : (b))
 #define floorf(x) floor(x)
@@ -93,9 +104,6 @@ void *alloca (size_t);
  * MINGW is also known as MSYS */
 #if defined(_MSC_VER) || defined(__MINGW32__)
 
-/* No langinfo in Windows or MinGW */
-#define nl_langinfo(X) ""
-
 /* strtok_r() is missing in Windows */
 char * strtok_r (char *s, const char *delim, char **saveptr);
 
@@ -122,7 +130,7 @@ void lg_mbrtowc(void);
 /*
  * CYGWIN prior to version 1.7 did not have UTF8 support, or wide
  * chars ... However, MS Visual C does, as does MinGW.  Since
- * some users have both cygwin and MinGW installed, crap out the 
+ * some users have both cygwin and MinGW installed, crap out the
  * UTF8 code only when MinGW is missing (and the CYGWIN version
  * is very old) XXX This code is dangerous and should be removed.
  */
@@ -193,7 +201,7 @@ int strncasecmp(const char *s1, const char *s2, size_t n);
 #define MAX(X,Y)  ( ((X) > (Y)) ? (X) : (Y))
 #endif
 
-/* Optimizations etc. that only gcc undersatands */
+/* Optimizations etc. that only gcc understands */
 #if __GNUC__ > 2
 #define GNUC_MALLOC __attribute__ ((malloc))
 #define GNUC_UNUSED __attribute__ ((unused))
@@ -203,7 +211,7 @@ int strncasecmp(const char *s1, const char *s2, size_t n);
 #endif
 
 /**
- * Return the length, in codepoints/glyphs, of the utf8-encoded 
+ * Return the length, in codepoints/glyphs, of the utf8-encoded
  * string.  This is needed when printing strings in a formatted way.
  */
 static inline size_t utf8_strlen(const char *s)
@@ -306,7 +314,7 @@ static inline const char * skip_utf8_upper(const char * s)
 /**
  * Return true if the initial upper-case letters of the
  * two input strings match. Comparison stops when
- * both srings descend to lowercase.
+ * both strings descend to lowercase.
  */
 static inline bool utf8_upper_match(const char * s, const char * t)
 {
