@@ -34,15 +34,21 @@ void free_X_nodes(X_node *);
 
 /* Connector utilities ... */
 Connector * connector_new(void);
-Connector * init_connector(Connector *c);
 void free_connectors(Connector *);
 
+static inline Connector * init_connector(Connector *c)
+{
+	c->hash = -1;
+	c->length_limit = UNLIMITED_LEN;
+	return c;
+}
+
 /* Connector-set utilities ... */
 Connector_set * connector_set_create(Exp *e);
 void connector_set_delete(Connector_set * conset);
 bool word_has_connector(Dict_node *, const char *, char);
 const char * word_only_connector(Dict_node *);
-bool match_in_connector_set(Connector_set*, Connector*, int dir);
+bool match_in_connector_set(Connector_set*, Connector*);
 
 
 /**
@@ -121,19 +127,18 @@ static inline int connector_hash(Connector * c)
  * hash function. Based on some tests, this seems to be an almost
  * "perfect" hash, in that almost all hash buckets have the same size!
  */
-static inline unsigned int pair_hash(unsigned int log2_table_size,
+static inline unsigned int pair_hash(unsigned int table_size,
                             int lw, int rw,
                             const Connector *le, const Connector *re,
                             unsigned int cost)
 {
-	unsigned int table_size = (1 << log2_table_size);
-	unsigned int i = 0;
+	unsigned int i;
 
 #if 0
  	/* hash function. Based on some tests, this seems to be
 	 * an almost "perfect" hash, in that almost all hash buckets
 	 * have the same size! */
-	i += 1 << cost;
+	i = 1 << cost;
 	i += 1 << (lw % (log2_table_size-1));
 	i += 1 << (rw % (log2_table_size-1));
 	i += ((unsigned int) le) >> 2;
