@@ -13,7 +13,7 @@
 
  /****************************************************************************
  *
- *   This is a simple example of the link parser API.  It similates most of
+ *   This is a simple example of the link parser API.  It simulates most of
  *   the functionality of the original link grammar parser, allowing sentences
  *   to be typed in either interactively or in "batch" mode (if -batch is
  *   specified on the command line, and stdin is redirected to a file).
@@ -35,9 +35,12 @@
 
 #include <errno.h>
 #include <locale.h>
+#include <stdlib.h>
+#include <string.h>
 
 /* Used for terminal resizing */
 #ifndef _WIN32
+#include <langinfo.h>
 #include <termios.h>
 #include <sys/ioctl.h>
 #include <fcntl.h>
@@ -54,14 +57,18 @@
 #include "../link-grammar/link-includes.h"
 #include "command-line.h"
 #include "lg_readline.h"
-#include "../link-grammar/utilities.h"     /* For MSVC portability */
+#ifdef USE_VITERBI
 #include "../viterbi/viterbi.h"
-#include "../link-grammar/error.h"
+#endif
 
 #define MAX_INPUT 1024
 #define DISPLAY_MAX 1024
 #define COMMENT_CHAR '%'  /* input lines beginning with this are ignored */
 
+#if !defined(MIN)
+#define MIN(X,Y)  ( ((X) < (Y)) ? (X) : (Y))
+#endif
+
 static int batch_errors = 0;
 static bool input_pending = false;
 static int verbosity = 0;
@@ -138,7 +145,7 @@ fget_input_string(FILE *in, FILE *out, Command_Options* copts)
 	}
 	input_pending = false;
 	if (pline) free(pline);
-	pline = lg_readline(prompt);
+	pline = lg_readline(prompt, copts->batch_mode);
 
 	return pline;
 
@@ -296,6 +303,7 @@ static void process_linkage(Linkage linkage, Command_Options* copts)
 		}
 		else
 		{
+			copts->display_constituents = 0;
 			fprintf(stderr, "Can't generate constituents.\n");
 			fprintf(stderr, "Constituent processing has been turned off.\n");
 		}
@@ -362,8 +370,26 @@ static int process_some_linkages(Sentence sent, Command_Options* copts)
 
 		linkage = linkage_create(i, sent, opts);
 
+		/* Currently, sat solver sets the linkage violation indication
+		 * only when it creates the linkage as a result of the above call. */
+		if ((sentence_num_violations(sent, i) > 0) &&
+			!copts->display_bad)
+		{
+			continue;
+		}
+
 		/* Currently, sat solver returns NULL when there ain't no more */
-		if (!linkage) break;
+		if (!linkage)
+		{
+			if (verbosity > 0)
+			{
+				if (0 == i)
+					fprintf(stdout, "No linkages found.\n");
+				else
+					fprintf(stdout, "No more linkages.\n");
+			}
+			break;
+		}
 
 		if (verbosity > 0)
 		{
@@ -452,8 +478,12 @@ static void batch_process_some_linkages(Label label,
 
 	if (there_was_an_error(label, sent, opts))
 	{
-		/* If linkages were found, print them */
-		if (sentence_num_linkages_found(sent) > 0) {
+		/* If linkages were found, print them.
+		 * The equality check takes care for the SAT solver, which, for
+		 * now, sets num_valid_linkages==0. In any case, if there are
+		 * actually no linkages, linkage_create() will return NULL, and
+		 * nothing will be processed. */
+		if (sentence_num_linkages_found(sent) >= 0) {
 			Linkage linkage = NULL;
 			/* If we found at least one good linkage, print it. */
 			if (sentence_num_valid_linkages(sent) > 0) {
@@ -528,11 +558,11 @@ static void setup_panic_parse_options(Parse_Options opts)
 	parse_options_set_min_null_count(opts, 1);
 	parse_options_set_max_null_count(opts, 100);
 	parse_options_set_max_parse_time(opts, 60);
-	parse_options_set_islands_ok(opts, 1);
+	parse_options_set_islands_ok(opts, true);
 	parse_options_set_short_length(opts, 12);
 	parse_options_set_all_short_connectors(opts, 1);
 	parse_options_set_linkage_limit(opts, 100);
-	parse_options_set_spell_guess(opts, false);
+	parse_options_set_spell_guess(opts, 0);
 }
 
 static void print_usage(char *str)
@@ -584,7 +614,7 @@ fail:
 	struct winsize ws;
 	int fd = fileno(stdout);
 
-	/* If ther is no controlling terminal, the fileno will fail. This
+	/* If there is no controlling terminal, the fileno will fail. This
 	 * seems to happen while building docker images, I don't know why.
 	 */
 	if (fd < 0) return;
@@ -699,6 +729,7 @@ int main(int argc, char * argv[])
 	parse_options_set_min_null_count(opts, 0);
 	parse_options_set_max_null_count(opts, 0);
 	parse_options_set_short_length(opts, 16);
+	parse_options_set_islands_ok(opts, true);
 
 	/* The English and Russian dicts use a cost of 2.7, which allows
 	 * regexes with a fractional cost of less than 1 to be used with
@@ -761,6 +792,10 @@ int main(int argc, char * argv[])
 		if (strncmp(input_string, "!file", 5) == 0)
 		{
 			char * filename = &input_string[6];
+			int fnlen = strlen(filename);
+
+			if ('\n' == filename[fnlen-1]) filename[fnlen-1] = '\0';
+
 			input_fh = fopen(filename, "r");
 			if (NULL == input_fh)
 			{
@@ -791,7 +826,7 @@ int main(int argc, char * argv[])
 				if (copts->batch_mode) batch_in_progress = true;
 			}
 		}
-		
+
 		if (copts->echo_on)
 		{
 			printf("%s\n", input_string);
@@ -850,7 +885,7 @@ int main(int argc, char * argv[])
 #endif
 
 			/* Try using a larger list of disjuncts */
-			/* XXX fixme: the lg_expand_disjunct_list() routine is not
+			/* XXX FIXME: the lg_expand_disjunct_list() routine is not
 			 * currently a part of the public API; it should be made so,
 			 * or this expansion idea should be abandoned... not sure which.
 			 */
@@ -906,6 +941,15 @@ int main(int argc, char * argv[])
 				/* print_total_time(opts); */
 				batch_errors++;
 				if (verbosity > 0) fprintf(stdout, "Entering \"panic\" mode...\n");
+				/* If the parser used was the SAT solver, set the panic parser to
+				 * it too.
+				 * FIXME? Currently, the SAT solver code is not too useful in
+				 * panic mode since it doesn't handle parsing with null words, so
+				 * using the regular parser in that case could be beneficial.
+				 * However, this currently causes a crash due to a memory
+				 * management mess. */
+				parse_options_set_use_sat_parser(copts->panic_opts,
+					parse_options_get_use_sat_parser(opts));
 				parse_options_reset_resources(copts->panic_opts);
 				parse_options_set_verbosity(copts->panic_opts, verbosity);
 				num_linkages = sentence_parse(sent, copts->panic_opts);
@@ -946,7 +990,7 @@ int main(int argc, char * argv[])
 				"%d error%s.\n", batch_errors, (batch_errors==1) ? "" : "s");
 	}
 
-	/* Free stuff, so that mem-leak detectors don't commplain. */
+	/* Free stuff, so that mem-leak detectors don't complain. */
 	command_options_delete(copts);
 	dictionary_delete(dict);
 	fget_input_string(NULL, NULL, NULL);
